//
// XMM Science Analysis System
//
// Giuseppe Vacanti (cosine) 
// September 2000
//

#include <Ippv.h>
#include <IppvTranslator.h>
#include <Tracer.h>
#include <Dal.h>
#include <Xmm.h>
#include <errstr.h>

template<class T> IppvTranslator<T>::IppvTranslator()
  : _map(map<T,string>())
{
  PROC
  ENDPROC
}

template<class T> IppvTranslator<T>::~IppvTranslator()
{
  PROC
  ENDPROC
}

template<class T> const string& IppvTranslator<T>::look_up(const T& t) const
{
  PROC
  if(_map.find(t) == _map.end())
    throw NoTranslationAvailable("No look up possible [" + _name + "]");
  RETURN _map[t];
}

template<class T> void IppvTranslator<T>::print(ostream& os) const
{
  PROC
  os << "Translator's name: " << _name << endl;
  for(map<T,string>::const_iterator k = _map.begin(); k != _map.end(); k++)
    os << (*k).first << " = " << (*k).second << endl;
  ENDPROC
}

template<class T> bool IppvTranslator<T>::has_look_up() const
{
  PROC
  RETURN _map.size() == 0 ? false : true;
}

//

template class IppvTranslator<int8>;
template class IppvTranslator<int16>;
template class IppvTranslator<uint16>;
template class IppvTranslator<int32>;
template class IppvTranslator<uint32>;
template class IppvTranslator<real32>;
template class IppvTranslator<real64>;
template class IppvTranslator<string>;

template<> EmosGainTranslator<string>::EmosGainTranslator(const string& ){
  _name = "EMOS Read Out Gain";
  _map["CHAINNORNOD0"] = "NORMAL";
  _map["CHAINNORNOD1"] = "NORMAL";
  _map["CHAIN/10NOD0"] = "LOW";
  _map["CHAIN/10NOD1"] = "LOW";

  errstr << "Setting up " + _name + " with the following:" << endl;
  for(map<string,string>::const_iterator k=_map.begin();
      k!=_map.end(); ++k)
    errstr << (*k).first << " -> " << (*k).second << endl;
  errstr <<Sas::message(Msg::AppLib,Msg::Noisy);

}
  
template<> EmosFilterTranslator<uint16>::EmosFilterTranslator(const string& prefix) {
  if(prefix == "E"){
    _name = "EMOS1 Filter";
    _map[0] = Xmm::toString(Xmm::CLOSED);
    _map[1334] = Xmm::toString(Xmm::OPEN);
    _map[267] = Xmm::toString(Xmm::THIN1);
    _map[534] = Xmm::toString(Xmm::THIN2);
    _map[801] = Xmm::toString(Xmm::MEDIUM);
    _map[1067] = Xmm::toString(Xmm::THICK);
    _map[1580] = Xmm::toString(Xmm::CAL_CLOSED);
    _map[1314] = Xmm::toString(Xmm::CAL_OPEN);
    _map[247] = Xmm::toString(Xmm::CAL_THIN1);
    _map[514] = Xmm::toString(Xmm::CAL_THIN2);
    _map[781] = Xmm::toString(Xmm::CAL_MEDIUM);
    _map[1047] = Xmm::toString(Xmm::CAL_THICK);
  } else if(prefix == "K"){
    _name = "EMOS2 Filter";
    _map[0] = Xmm::toString(Xmm::CLOSED);
    _map[1332] = Xmm::toString(Xmm::OPEN);
    _map[266] = Xmm::toString(Xmm::THIN1);
    _map[533] = Xmm::toString(Xmm::THIN2);
    _map[799] = Xmm::toString(Xmm::MEDIUM);
    _map[1066] = Xmm::toString(Xmm::THICK);
    _map[1580] = Xmm::toString(Xmm::CAL_CLOSED);
    _map[1312] = Xmm::toString(Xmm::CAL_OPEN);
    _map[246] = Xmm::toString(Xmm::CAL_THIN1);
    _map[513] = Xmm::toString(Xmm::CAL_THIN2);
    _map[779] = Xmm::toString(Xmm::CAL_MEDIUM);
    _map[1046] = Xmm::toString(Xmm::CAL_THICK);
  } else
    errstr << "Emos filter translator faled." << Sas::error("CannotCreateTranslator");
  errstr << "Setting up " + _name + " with the following:" << endl;
  for(map<uint16,string>::const_iterator k=_map.begin();
      k!=_map.end(); ++k)
    errstr << (*k).first << " -> " << (*k).second << endl;
  errstr <<Sas::message(Msg::AppLib,Msg::Noisy);

}

template<> OmFilterTranslator<int16>::OmFilterTranslator() {
  _name = "OM Filter";
  _map[1200] = Xmm::toString(Xmm::BLOCKED);
  _map[1400] = Xmm::toString(Xmm::V);
  _map[1600] = Xmm::toString(Xmm::MAGNI);
  _map[1800] = Xmm::toString(Xmm::U);
  _map[2000] = Xmm::toString(Xmm::B);
  _map[0] = Xmm::toString(Xmm::WHITE);
  _map[200] = Xmm::toString(Xmm::GRISM2);
  _map[400] = Xmm::toString(Xmm::UVW1);
  _map[600] = Xmm::toString(Xmm::UVM2);
  _map[800] = Xmm::toString(Xmm::UVW2);
  _map[1000] = Xmm::toString(Xmm::GRISM1);
  _map[2100] = Xmm::toString(Xmm::BARRED_U);
  _map[2200] = "PositionLost";
  errstr << "Setting up " + _name + " with the following:" << endl;
  for(map<int16,string>::const_iterator k=_map.begin();
      k!=_map.end(); ++k)
    errstr << (*k).first << " -> " << (*k).second << endl;
  errstr <<Sas::message(Msg::AppLib,Msg::Noisy);
}

template<> EpnFilterWheelTranslator<string>::EpnFilterWheelTranslator() {
  _name = "EPN Filter Wheel";
  // Fix typos in HK
//    _map["OPEN"] = "Open";
//    _map["FILTRD-THICK"] = "Filter_D_thick";
//    _map["FILTC-MEDIUM"] = "Filter_C_medium";
//    _map["FILTRB-THIN"] = "Filter_B_thin";
//    _map["FILTRA-THIN1"] = "Filter_A_thin";
//    _map["CLOSE"] = "Closed";
//    _map["NO STOP POS"] = "CalibrationPosition";
//    _map["NO STOP POS."] = "CalibrationPosition";
  _map["OPEN"] = Xmm::toString(Xmm::OPEN);
  _map["FILTRD-THICK"] = Xmm::toString(Xmm::THICK);
  _map["FILTC-MEDIUM"] = Xmm::toString(Xmm::MEDIUM);
  // FIXME: how many typos are there?? Is it THIN or THIN2??
  _map["FILTRB-THIN"] = Xmm::toString(Xmm::THIN2);
  _map["FILTRB-THIN2"] = Xmm::toString(Xmm::THIN2);
  _map["FILTRA-THIN1"] = Xmm::toString(Xmm::THIN1);
  _map["CLOSE"] = Xmm::toString(Xmm::CLOSED);
  _map["NO STOP POS"] = Xmm::toString(Xmm::FILTER_NONE);
  _map["NO STOP POS."] = Xmm::toString(Xmm::FILTER_NONE);

  errstr << "Setting up " + _name + " with the following:" << endl;
  for(map<string,string>::const_iterator k=_map.begin();
      k!=_map.end(); ++k)
    errstr << (*k).first << " -> " << (*k).second << endl;
  errstr <<Sas::message(Msg::AppLib,Msg::Noisy);
}

template<> EpnFilterTranslator<real32>::EpnFilterTranslator() {
  _name = "EPN Filter";

  // These are the filter wheel positions in degrees.
  // Row 1: filter positions
  // Row 2: filter CAL positions (4.5 degrees behind)
   real32 v[] = { 55.,  115.15, 175.4, 235.6, 295.4, -5.6 ,
                 50.5, 110.65, 170.9, 231.1, 290.9, -10.1};
  unsigned n = sizeof(v)/sizeof(real32);
  vector<real32> vv(v,v+n);
  real32 t = 4.5/2.; // Tolerance around each filter position

  vector<string> vn(n);
  vn[0] = Xmm::toString(Xmm::CLOSED);
  vn[1] = Xmm::toString(Xmm::THIN1);
  vn[2] = Xmm::toString(Xmm::THIN2);
  vn[3] = Xmm::toString(Xmm::MEDIUM);
  vn[4] = Xmm::toString(Xmm::THICK);
  vn[5] = Xmm::toString(Xmm::OPEN);
  vn[6] = Xmm::toString(Xmm::CAL_CLOSED);
  vn[7] = Xmm::toString(Xmm::CAL_THIN1);
  vn[8] = Xmm::toString(Xmm::CAL_THIN2);
  vn[9] = Xmm::toString(Xmm::CAL_MEDIUM);
  vn[10] = Xmm::toString(Xmm::CAL_THICK);
  vn[11] = Xmm::toString(Xmm::CAL_OPEN);

  for(unsigned k=0; k<n; ++k)
    _mymap[pair<real32,real32>(vv[k]-t, vv[k]+t)] = vn[k];    

  errstr << "Setting up " + _name + " with the following:" << endl;
  for(map<pair<real32,real32>,string>::const_iterator k=_mymap.begin();
      k!=_mymap.end(); ++k)
    errstr << "[" << (*k).first.first << "," << (*k).first.second << "]"
	   << " -> " << (*k).second << endl;
  errstr <<Sas::message(Msg::AppLib,Msg::Noisy);

}

template<> const string& EpnFilterTranslator<real32>::look_up(const real32& t) const {
  bool good = false;
  typedef map<pair<real32,real32>, string>::const_iterator I;
  I match;
  unsigned nmatch = 0;
  for(I k=_mymap.begin();
      k!=_mymap.end(); ++k){
    const pair<real32,real32> &p = (*k).first;
    if(t> p.first && t< p.second){
      match = k;
      good = true;
      ++nmatch;
    }
  }
  if(nmatch != 1){
    good = false;
    errstr << "Possible matches for PN filter: " << nmatch << Sas::message(Msg::AppLib, Msg::Silent);
  }
  if(good)
    return (*match).second;
  else{
    throw NoTranslationAvailable("No look up possible [" + _name + "]");
  }
}

template<> void EpnFilterTranslator<real32>::print(ostream& os) const {
  PROC
  os << "Translator's name: " << _name << endl;
  typedef map<pair<real32,real32>, string>::const_iterator I;
  for(I k=_mymap.begin();
      k!=_mymap.end(); ++k){
    const pair<real32,real32> &p = (*k).first;
    os << "(" << p.first << "," << p.second << ") = " << (*k).second << endl;
  }
  ENDPROC
}
